@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2011-2022 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
+ * Copyright (c) 2011-2024 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -85,11 +85,12 @@ static void kill_child(int)  __attribute__((__noreturn__));
  Live-mode related functions 
  ****************************/
 
-/* Status */
+/* Live status (complements main_status) */
 static struct {
-    int fd;                      /* current file descriptor
+    char *entry_path;            /* current fts(3) entry path */
+    int fd;                      /* current output file descriptor
                                     (if option '-o' used) */
-    char *filename;              /* current file name */
+    char *filename;              /* current output file name */
     pnum_t partition_index;      /* current partition number */
     fsize_t partition_size;      /* current partition size */
     fnum_t partition_num_files;  /* number of files in current partition */
@@ -99,6 +100,7 @@ static struct {
                                     else 1 */
     pid_t child_pid;
 } live_status = {
+    NULL,
     STDOUT_FILENO,
     NULL,
     0,
@@ -129,13 +131,20 @@ kill_child(int sig)
      else returns 1 */
 int
 fpart_hook(const char *cmd, const struct program_options *options,
-    const char *live_filename, const pnum_t *live_partition_index,
-    const fsize_t *live_partition_size, const fnum_t *live_num_files,
-    const int partition_errno)
+    const struct program_status *status, const char *live_filename,
+    const pnum_t *live_partition_index, const fsize_t *live_partition_size,
+    const fnum_t *live_partition_num_files, const int live_partition_errno)
 {
     assert(cmd != NULL);
     assert(options != NULL);
-    assert(partition_errno >= 0);
+    assert(status != NULL);
+    if(cmd != options->post_run_hook) {
+        /* those arguments are only relevant in *-part hooks */
+        assert(live_partition_index != NULL);
+        assert(live_partition_size != NULL);
+        assert(live_partition_num_files != NULL);
+        assert(live_partition_errno >= 0);
+    }
 
     int retval = 0;
 
@@ -144,18 +153,24 @@ fpart_hook(const char *cmd, const struct program_options *options,
     char *env_fpart_partfilename_name = "FPART_PARTFILENAME";
     char *env_fpart_partnumber_name = "FPART_PARTNUMBER";
     char *env_fpart_partsize_name = "FPART_PARTSIZE";
+    char *env_fpart_totalsize_name = "FPART_TOTALSIZE";
     char *env_fpart_partnumfiles_name = "FPART_PARTNUMFILES";
+    char *env_fpart_totalnumfiles_name = "FPART_TOTALNUMFILES";
     char *env_fpart_parterrno_name = "FPART_PARTERRNO";
     char *env_fpart_pid_name = "FPART_PID";
+    char *env_fpart_totalnumparts_name = "FPART_TOTALNUMPARTS";
 
     /* env variables' values */
     char *env_fpart_hooktype_string = NULL;
     char *env_fpart_partfilename_string = NULL;
     char *env_fpart_partnumber_string = NULL;
     char *env_fpart_partsize_string = NULL;
+    char *env_fpart_totalsize_string = NULL;
     char *env_fpart_partnumfiles_string = NULL;
+    char *env_fpart_totalnumfiles_string = NULL;
     char *env_fpart_parterrno_string = NULL;
     char *env_fpart_pid_string = NULL;
+    char *env_fpart_totalnumparts_string = NULL;
 
     /* XXX As setenv(3)/unsetenv(3) are not available on all platforms, and there does not
     seem to be a standard way of unsetting variables through putenv(3), clone current
@@ -171,10 +186,6 @@ fpart_hook(const char *cmd, const struct program_options *options,
 
     /* determine the kind of hook we are in */
     if(cmd == options->pre_part_hook) {
-        assert(live_partition_index != NULL);
-        assert(live_partition_size != NULL);
-        assert(live_num_files != NULL);
-
         if(options->verbose >= OPT_VERBOSE)
             fprintf(stderr, "Executing pre-part #%ju hook: '%s'\n",
                 adapt_partition_index(*live_partition_index, options), cmd);
@@ -194,10 +205,6 @@ fpart_hook(const char *cmd, const struct program_options *options,
         }
     }
     else if(cmd == options->post_part_hook) {
-        assert(live_partition_index != NULL);
-        assert(live_partition_size != NULL);
-        assert(live_num_files != NULL);
-
         if(options->verbose >= OPT_VERBOSE)
             fprintf(stderr, "Executing post-part #%ju hook: '%s'\n",
                 adapt_partition_index(*live_partition_index, options), cmd);
@@ -216,6 +223,24 @@ fpart_hook(const char *cmd, const struct program_options *options,
             goto cleanup;
         }
     }
+    else /* cmd == options->post_run_hook */ {
+        if(options->verbose >= OPT_VERBOSE)
+            fprintf(stderr, "Executing post-run hook: '%s'\n", cmd);
+
+        /* FPART_HOOKTYPE (post-run) */
+        malloc_size = strlen(env_fpart_hooktype_name) + 1 +
+            strlen("post-run") + 1;
+        if_not_malloc(env_fpart_hooktype_string, malloc_size,
+            retval = 1;
+            goto cleanup;
+        )
+        snprintf(env_fpart_hooktype_string, malloc_size, "%s=%s",
+            env_fpart_hooktype_name, "post-run");
+        if(push_env(env_fpart_hooktype_string, &envp) != 0) {
+            retval = 1;
+            goto cleanup;
+        }
+    }
 
     /* FPART_PARTFILENAME */
     if(live_filename != NULL) {
@@ -265,36 +290,66 @@ fpart_hook(const char *cmd, const struct program_options *options,
         }
     }
 
+    /* FPART_TOTALSIZE */
+    malloc_size = strlen(env_fpart_totalsize_name) + 1 +
+        get_num_digits(status->total_size) + 1;
+    if_not_malloc(env_fpart_totalsize_string, malloc_size,
+        retval = 1;
+        goto cleanup;
+    )
+    snprintf(env_fpart_totalsize_string, malloc_size, "%s=%ju",
+        env_fpart_totalsize_name, status->total_size);
+    if(push_env(env_fpart_totalsize_string, &envp) != 0) {
+        retval = 1;
+        goto cleanup;
+    }
+
     /* FPART_PARTNUMFILES */
-    if(live_num_files != NULL) {
+    if(live_partition_num_files != NULL) {
         malloc_size = strlen(env_fpart_partnumfiles_name) + 1 +
-            get_num_digits(*live_num_files) + 1;
+            get_num_digits(*live_partition_num_files) + 1;
         if_not_malloc(env_fpart_partnumfiles_string, malloc_size,
             retval = 1;
             goto cleanup;
         )
         snprintf(env_fpart_partnumfiles_string, malloc_size, "%s=%ju",
-            env_fpart_partnumfiles_name, *live_num_files);
+            env_fpart_partnumfiles_name, *live_partition_num_files);
         if(push_env(env_fpart_partnumfiles_string, &envp) != 0) {
             retval = 1;
             goto cleanup;
         }
     }
 
-    /* FPART_PARTERRNO */
-    malloc_size = strlen(env_fpart_parterrno_name) + 1 +
-        get_num_digits(partition_errno) + 1;
-    if_not_malloc(env_fpart_parterrno_string, malloc_size,
+    /* FPART_TOTALNUMFILES */
+    malloc_size = strlen(env_fpart_totalnumfiles_name) + 1 +
+        get_num_digits(status->total_num_files) + 1;
+    if_not_malloc(env_fpart_totalnumfiles_string, malloc_size,
         retval = 1;
         goto cleanup;
     )
-    snprintf(env_fpart_parterrno_string, malloc_size, "%s=%d",
-        env_fpart_parterrno_name, partition_errno);
-    if(push_env(env_fpart_parterrno_string, &envp) != 0) {
+    snprintf(env_fpart_totalnumfiles_string, malloc_size, "%s=%ju",
+        env_fpart_totalnumfiles_name, status->total_num_files);
+    if(push_env(env_fpart_totalnumfiles_string, &envp) != 0) {
         retval = 1;
         goto cleanup;
     }
 
+    /* FPART_PARTERRNO */
+    if(cmd != options->post_run_hook) {
+        malloc_size = strlen(env_fpart_parterrno_name) + 1 +
+            get_num_digits(live_partition_errno) + 1;
+        if_not_malloc(env_fpart_parterrno_string, malloc_size,
+            retval = 1;
+            goto cleanup;
+        )
+        snprintf(env_fpart_parterrno_string, malloc_size, "%s=%d",
+            env_fpart_parterrno_name, live_partition_errno);
+        if(push_env(env_fpart_parterrno_string, &envp) != 0) {
+            retval = 1;
+            goto cleanup;
+        }
+    }
+
     /* FPART_PID */
     pid_t fpart_pid = getpid();
     malloc_size = strlen(env_fpart_pid_name) + 1 +
@@ -310,6 +365,20 @@ fpart_hook(const char *cmd, const struct program_options *options,
         goto cleanup;
     }
 
+    /* FPART_TOTALNUMPARTS */
+    malloc_size = strlen(env_fpart_totalnumparts_name) + 1 +
+        get_num_digits(status->total_num_parts) + 1;
+    if_not_malloc(env_fpart_totalnumparts_string, malloc_size,
+        retval = 1;
+        goto cleanup;
+    )
+    snprintf(env_fpart_totalnumparts_string, malloc_size, "%s=%ju",
+        env_fpart_totalnumparts_name, status->total_num_parts);
+    if(push_env(env_fpart_totalnumparts_string, &envp) != 0) {
+        retval = 1;
+        goto cleanup;
+    }
+
     /* fork child process */
     int child_status = 0;
     switch(live_status.child_pid = fork()) {
@@ -385,10 +454,18 @@ cleanup:
         free(env_fpart_partnumber_string);
     if(env_fpart_partsize_string != NULL)
         free(env_fpart_partsize_string);
+    if(env_fpart_totalsize_string != NULL)
+        free(env_fpart_totalsize_string);
     if(env_fpart_partnumfiles_string != NULL)
         free(env_fpart_partnumfiles_string);
+    if(env_fpart_totalnumfiles_string != NULL)
+        free(env_fpart_totalnumfiles_string);
+    if(env_fpart_parterrno_string != NULL)
+        free(env_fpart_parterrno_string);
     if(env_fpart_pid_string != NULL)
         free(env_fpart_pid_string);
+    if(env_fpart_totalnumparts_string != NULL)
+        free(env_fpart_totalnumparts_string);
     return (retval);
 }
 
@@ -398,16 +475,21 @@ cleanup:
    - returns (-1) if error */
 int
 handle_file_entry(struct file_entry **head, char *path, fsize_t size,
-    int entry_errno, struct program_options *options)
+    int entry_errno, struct program_options *options,
+    struct program_status *status)
 {
     assert(options != NULL);
+    assert(status != NULL);
     assert(entry_errno >= 0);
 
+    /* overload and round size */
+    size = round_num(size + options->overload_size, options->round_size);
+
     if(options->live_mode == OPT_LIVEMODE)
-        return (live_print_file_entry(path, size, entry_errno, options));
+        return (live_print_file_entry(path, size, entry_errno, options, status));
     else
         /* XXX propagate (and exploit) entry_errno in non-live mode too ? */
-        return (add_file_entry(head, path, size, options));
+        return (add_file_entry(head, path, size, options, status));
 }
 
 /* Display a single entry line */
@@ -431,7 +513,7 @@ display_file_entry(pnum_t partition_index, const fsize_t entry_size,
    - returns (-1) if error */
 int
 live_print_file_entry(char *path, fsize_t size, int entry_errno,
-    struct program_options *options)
+    struct program_options *options, struct program_status *status)
 {
 /* split states */
 #define SPLIT_NONE 0
@@ -439,9 +521,10 @@ live_print_file_entry(char *path, fsize_t size, int entry_errno,
 #define SPLIT_END  2
 
     assert(path != NULL);
+    assert(entry_errno >= 0);
     assert(options != NULL);
     assert(options->live_mode == OPT_LIVEMODE);
-    assert(entry_errno >= 0);
+    assert(status != NULL);
 
     char *out_template = options->out_filename;
     char *ln_term = (options->out_zero == OPT_OUT0) ? "\0" : "\n";
@@ -449,12 +532,12 @@ live_print_file_entry(char *path, fsize_t size, int entry_errno,
 
     /* option -S: skip files bigger than maximum partition size (option -s)
        and print them to stdout in hardcoded pseudo-partition 'S' ('S'kipped).
-       Preloading and overloading are used here too */
+       Preloading and overloading are already done at that step */
     if(options->skip_big == OPT_SKIPBIG) {
-        fsize_t needed_part_size = options->preload_size +
-            round_num(size + options->overload_size, options->round_size);
+        fsize_t needed_part_size = options->preload_size + size;
         if(needed_part_size > options->max_size) {
-            display_file_entry(0, size, path, ENTRY_DISPLAY_TYPE_SKIPPED); /* partition_index irrelevant here */
+            /* partition_index irrelevant here */
+            display_file_entry(0, size, path, ENTRY_DISPLAY_TYPE_SKIPPED);
             fflush(stdout);
             return (1);
         }
@@ -464,9 +547,14 @@ live_print_file_entry(char *path, fsize_t size, int entry_errno,
     if(live_status.partition_num_files == 0) {
 start_part:
         /* very first pass of first partition, preload first partition */
-        if(live_status.partition_index == 0)
+        if(live_status.partition_index == 0) {
             live_status.partition_size = options->preload_size;
 
+            /* count part in */
+            status->total_size += options->preload_size;
+            status->total_num_parts++;
+        }
+
         if(out_template != NULL) {
             /* compute live_status.filename "out_template.i\0" */
             size_t malloc_size = strlen(out_template) + 1 +
@@ -480,7 +568,7 @@ start_part:
 
         /* execute pre-partition hook */
         if(options->pre_part_hook != NULL) {
-            if(fpart_hook(options->pre_part_hook, options,
+            if(fpart_hook(options->pre_part_hook, options, status,
                 live_status.filename,
                 &live_status.partition_index,
                 &live_status.partition_size,
@@ -515,9 +603,13 @@ start_part:
     }
 
     /* count file in */
-    live_status.partition_size +=
-        round_num(size + options->overload_size, options->round_size);
+    live_status.partition_size += size;
     live_status.partition_num_files++;
+    /* in live mode, global counters are not updated from handle_file_entry(),
+       because that would make them updated *after* hook calls, rendering
+       FPART_TOTALSIZE and FPART_TOTALNUMFILES variables erroneous */
+    status->total_size += size;
+    status->total_num_files++;
     if(entry_errno != 0)
         live_status.partition_errno = entry_errno;
 
@@ -532,7 +624,7 @@ start_part:
         if((write(live_status.fd, path, to_write) != (ssize_t)to_write) ||
             (write(live_status.fd, ln_term, 1) != 1)) {
             fprintf(stderr, "%s\n", strerror(errno));
-            /* do not close(livefd) and free(live_status.filename) here because
+            /* do not close(live_status.fd) and free(live_status.filename) here because
                it will be useful and free'd in uninit_file_entries() below */
             return (-1);
         }
@@ -549,6 +641,59 @@ start_part:
             (live_status.partition_size >= options->max_size)) ||
         (split == SPLIT_END)) {
 end_part:
+        /* display parent directories if requested */
+        if(options->add_parents == OPT_ADDPARENTS) {
+            char *parent = NULL;
+            char *grandparent = NULL;
+
+            if(options->verbose >= OPT_VVERBOSE)
+                fprintf(stderr, "Adding parents to finalize partition...\n");
+
+            /* allocate and compute parent paths,
+               stop when live_status.entry_path has been reached */
+            parent = parent_path(path, 1);
+            while((parent != NULL) &&
+                  (parent[0] != '\0') &&
+                  (strstr(parent, live_status.entry_path) != NULL)) {
+                if(out_template == NULL) {
+                    /* no template provided, just print parent to stdout */
+                    display_file_entry(adapt_partition_index(live_status.partition_index, options),
+                        0, parent, ENTRY_DISPLAY_TYPE_STANDARD);
+                }
+                else {
+                    /* print to fd */
+                    size_t to_write = strlen(parent);
+                    if((write(live_status.fd, parent, to_write) != (ssize_t)to_write) ||
+                        (write(live_status.fd, ln_term, 1) != 1)) {
+                        fprintf(stderr, "%s\n", strerror(errno));
+                        free(parent);
+                        /* do not close(live_status.fd) and free(live_status.filename) here because
+                           it will be useful and free'd in uninit_file_entries() below */
+                        return (-1);
+                    }
+                }
+
+                /* display added parent */
+                if(options->verbose >= OPT_VVERBOSE)
+                    fprintf(stderr, "%s\n", parent);
+
+                /* root reached */
+                if((parent[0] == '/') && (parent[1] == '\0'))
+                    break;
+
+                /* next parent */
+                grandparent = parent;
+                parent = parent_path(grandparent, 1);
+                free(grandparent);
+            }
+            if(parent == NULL)
+                /* do not close(live_status.fd) and free(live_status.filename) here because
+                   it will be useful and free'd in uninit_file_entries() below */
+                return (-1);
+
+            free(parent);
+        }
+
         /* display added partition */
         if(options->verbose >= OPT_VERBOSE)
             display_partition_summary(adapt_partition_index(live_status.partition_index, options),
@@ -563,7 +708,7 @@ end_part:
 
         /* execute post-partition hook */
         if(options->post_part_hook != NULL) {
-            if(fpart_hook(options->post_part_hook, options,
+            if(fpart_hook(options->post_part_hook, options, status,
                 live_status.filename,
                 &live_status.partition_index,
                 &live_status.partition_size,
@@ -583,6 +728,10 @@ end_part:
         live_status.partition_num_files = 0;
         live_status.partition_errno = 0;
 
+        /* count part in */
+        status->total_size += options->preload_size;
+        status->total_num_parts++;
+
         /* create another partition with single entry */
         if (split == SPLIT_DO) {
             split = SPLIT_END;
@@ -600,17 +749,19 @@ end_part:
 /* Add a file entry to a double-linked list of file_entries
    - if head is NULL, creates a new file entry ; if not, chains a new file
      entry to it
+   - increments *status counters
    - returns (0) if entry has been added
    - returns (-1) if error
    - returns with head set to the newly added element */
 int
 add_file_entry(struct file_entry **head, char *path, fsize_t size,
-    struct program_options *options)
+    struct program_options *options, struct program_status *status)
 {
     assert(head != NULL);
     assert(path != NULL);
     assert(options != NULL);
     assert(options->live_mode == OPT_NOLIVEMODE);
+    assert(status != NULL);
 
     struct file_entry **current = head; /* current file_entry pointer address */
     struct file_entry *previous = NULL; /* previous file_entry pointer */
@@ -635,8 +786,7 @@ add_file_entry(struct file_entry **head, char *path, fsize_t size,
         return (-1);
     )
     snprintf((*current)->path, malloc_size, "%s", path);
-    (*current)->size = size + options->overload_size;
-    (*current)->size = round_num((*current)->size, options->round_size);
+    (*current)->size = size;
 
     /* set current file entry's index and pointers */
     (*current)->partition_index = 0;    /* set during dispatch */
@@ -647,6 +797,10 @@ add_file_entry(struct file_entry **head, char *path, fsize_t size,
     if(previous != NULL)
         previous->nextp = *current;
 
+    /* count file in */
+    status->total_size += size;
+    status->total_num_files++;
+
     /* display added filename */
     if(options->verbose >= OPT_VVERBOSE)
         fprintf(stderr, "%s\n", (*current)->path);
@@ -690,17 +844,17 @@ fts_dirsfirst(const FTSENT * const *a, const FTSENT * const *b)
 /* Initialize a double-linked list of file_entries from a path
    - file_path may be a file or directory
    - if head is NULL, creates a new list ; if not, chains a new list to it
-   - increments *count with the number of files found
+   - increments *status counters
    - returns != 0 if critical error
    - returns with head set to the last element added */
 int
-init_file_entries(char *file_path, struct file_entry **head, fnum_t *count,
-    struct program_options *options)
+init_file_entries(char *file_path, struct file_entry **head,
+	struct program_options *options, struct program_status *status)
 {
     assert(file_path != NULL);
     assert(head != NULL);
-    assert(count != NULL);
     assert(options != NULL);
+    assert(status != NULL);
 
     /* prepare fts */
     FTS *ftsp = NULL;
@@ -719,7 +873,8 @@ init_file_entries(char *file_path, struct file_entry **head, fnum_t *count,
     int (*fts_sortfuncp)(const FTSENT * const *, const FTSENT * const *);
 #endif
     if((options->dirs_only == OPT_DIRSONLY) ||
-       (options->leaf_dirs == OPT_LEAFDIRS))
+       (options->leaf_dirs == OPT_LEAFDIRS) ||
+       (options->add_parents == OPT_ADDPARENTS))
         fts_sortfuncp = &fts_dirsfirst;
     else
         fts_sortfuncp = NULL;
@@ -744,6 +899,20 @@ init_file_entries(char *file_path, struct file_entry **head, fnum_t *count,
                                            (see fts(3)), but fts_read_errno is
                                            needed in FTS_DP */
 
+    /* keep a copy of current entry path in live mode */
+    if(options->live_mode == OPT_LIVEMODE) {
+        /* free() previous entry path if necessary */
+        if(live_status.entry_path != NULL)
+            free(live_status.entry_path);
+
+        /* copy current entry path */
+        size_t malloc_size = strlen(file_path) + 1;
+        if_not_malloc(live_status.entry_path, malloc_size,
+            return (0);
+        )
+        snprintf(live_status.entry_path, malloc_size, "%s", file_path);
+    }
+
     while((p = fts_read(ftsp)) != NULL) {
         if(options->verbose >= OPT_VVVERBOSE) {
             fprintf(stderr, "%s(%s): fts_info=%d, fts_errno=%d\n", __func__,
@@ -896,11 +1065,8 @@ add_directory:
                     /* else, trust curdir_size and leave it untouched. */
 
                     /* add or display it */
-                    int handled = handle_file_entry(head, curdir_entry_path,
-                        curdir_size, fts_read_errno, options);
-                    if(handled == 0)
-                        (*count)++;
-                    else if(handled < 0) {
+                    if(handle_file_entry(head, curdir_entry_path, curdir_size,
+                        fts_read_errno, options, status) < 0) {
                         fprintf(stderr, "%s(): cannot add file entry\n",
                             __func__);
                         free(curdir_entry_path);
@@ -1008,12 +1174,9 @@ two-pass check to handle include and exclude options properly. */
                     continue;
 
                 /* add or display it */
-                int handled = handle_file_entry(head, p->fts_path,
-                    curfile_size, 0, options); /* fts_read_errno is always 0
-                                                  here, so harcode it */
-                if(handled == 0)
-                    (*count)++;
-                else if(handled < 0) {
+                if(handle_file_entry(head, p->fts_path,
+                       curfile_size, 0 /* fts_read_errno is always 0 here,
+                       so hardcode it */, options, status) < 0) {
                     fprintf(stderr, "%s(): cannot add file entry\n", __func__);
                     fts_close(ftsp);
                     return (1);
@@ -1037,9 +1200,11 @@ two-pass check to handle include and exclude options properly. */
 
 /* Un-initialize a double-linked list of file_entries */
 void
-uninit_file_entries(struct file_entry *head, struct program_options *options)
+uninit_file_entries(struct file_entry *head, struct program_options *options,
+    struct program_status *status)
 {
     assert(options != NULL);
+    assert(status != NULL);
 
     /* be sure to start from last file entry */
     fastfw_list(head);
@@ -1074,7 +1239,7 @@ uninit_file_entries(struct file_entry *head, struct program_options *options)
         /* execute last post-partition hook */
         if((options->post_part_hook != NULL) &&
             (live_status.partition_num_files > 0)) {
-            if(fpart_hook(options->post_part_hook, options,
+            if(fpart_hook(options->post_part_hook, options, status,
                 live_status.filename,
                 &live_status.partition_index,
                 &live_status.partition_size,
@@ -1088,6 +1253,22 @@ uninit_file_entries(struct file_entry *head, struct program_options *options)
             live_status.filename = NULL;
         }
 
+        if(live_status.entry_path != NULL) {
+            free(live_status.entry_path);
+            live_status.entry_path = NULL;
+        }
+
+        /* execute post-run hook */
+        if(options->post_run_hook != NULL) {
+            if(fpart_hook(options->post_run_hook, options, status,
+                NULL,   /* remaining arguments are irrelevant here */
+                NULL,
+                NULL,
+                NULL,
+                0) != 0)
+                live_status.exit_summary = 1;
+        }
+
         /* print hooks' exit codes summary */
         if((options->verbose >= OPT_VERBOSE) && (live_status.exit_summary != 0))
             fprintf(stderr, "Warning: at least one hook exited with error !\n");
