@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2011-2022 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
+ * Copyright (c) 2011-2024 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -42,12 +42,45 @@
 /* fprintf(3), fopen(3), fclose(3), fgets(3), foef(3) */
 #include <stdio.h>
 
-/* getopt(3) */
+/* getopt(3) / getopt_long(3) */
 #include <unistd.h>
-#if !defined(__SunOS_5_9)
+#if defined(HAVE_GETOPT_H)
 #include <getopt.h>
 #endif
 
+/* Short options */
+#if defined(_HAS_FNM_CASEFOLD)
+#define OPTIONS "+hVn:f:s:i:ao:0ePvlby:Y:x:X:zZd:DELSw:W:R:p:q:r:"
+#else
+#define OPTIONS "+hVn:f:s:i:ao:0ePvlby:x:zZd:DELSw:W:R:p:q:r:"
+#endif
+
+/* Long options */
+#if defined(HAVE_GETOPT_LONG)
+#define GETOPT getopt_long
+static struct option long_options[] =
+{
+    { "help",           no_argument,        NULL, 'h' },
+    { "version",        no_argument,        NULL, 'V' },
+    { "parts",          required_argument,  NULL, 'n' },
+    { "files",          required_argument,  NULL, 'f' },
+    { "size",           required_argument,  NULL, 's' },
+    { "arbitrary",      no_argument,        NULL, 'a' },
+    { "verbose",        no_argument,        NULL, 'v' },
+    { "include",        required_argument,  NULL, 'y' },
+    { "exclude",        required_argument,  NULL, 'x' },
+    { "leaf-dirs",      no_argument,        NULL, 'D' },
+    { "dirs-only",      no_argument,        NULL, 'E' },
+    { "live",           no_argument,        NULL, 'L' },
+    { "pre-part-cmd",   required_argument,  NULL, 'w' },
+    { "post-part-cmd",  required_argument,  NULL, 'W' },
+    { "post-run-cmd",  required_argument,  NULL, 'R' },
+    { NULL, 0, NULL, 0 }
+};
+#else
+#define GETOPT getopt
+#endif
+
 /* strlen(3) */
 #include <string.h>
 
@@ -65,7 +98,7 @@ static void
 version(void)
 {
     fprintf(stderr, "fpart v" FPART_VERSION "\n"
-        "Copyright (c) 2011-2022 "
+        "Copyright (c) 2011-2024 "
         "Ganael LAPLANCHE <ganael.laplanche@martymac.org>\n"
         "WWW: http://contribs.martymac.org\n");
     fprintf(stderr, "Build options: debug=");
@@ -78,6 +111,12 @@ version(void)
     fprintf(stderr, "embedded\n");
 #else
     fprintf(stderr, "system\n");
+#endif
+    fprintf(stderr, "Long options are ");
+#if defined(HAVE_GETOPT_LONG)
+    fprintf(stderr, "supported\n");
+#else
+    fprintf(stderr, "not supported\n");
 #endif
 }
 
@@ -85,83 +124,107 @@ version(void)
 static void
 usage(void)
 {
+    fprintf(stderr, "fpart v" FPART_VERSION " - Sort and pack files into "
+        "partitions\n");
     fprintf(stderr, "Usage: fpart [OPTIONS] -n num | -f files | -s size "
         "[FILE or DIR...]\n");
-    fprintf(stderr, "Sort and pack files into partitions.\n");
     fprintf(stderr, "\n");
     fprintf(stderr, "General options:\n");
-    fprintf(stderr, "  -h\tthis help\n");
-    fprintf(stderr, "  -V\tprint version\n");
+    fprintf(stderr, "  -h, --help           print this help\n");
+    fprintf(stderr, "  -V, --version        print version\n");
     fprintf(stderr, "\n");
     fprintf(stderr, "Partition control:\n");
-    fprintf(stderr, "  -n\tpack files into <num> partitions\n");
-    fprintf(stderr, "  -f\tlimit partitions to <files> files or directories\n");
-    fprintf(stderr, "  -s\tlimit partitions to <size> bytes\n");
+    fprintf(stderr, "  -n, --parts          pack files into <num> "
+        "partitions\n");
+    fprintf(stderr, "  -f, --files          limit partitions to <files> files "
+        "or directories\n");
+    fprintf(stderr, "  -s, --size           limit partitions to <size> "
+        "bytes\n");
     fprintf(stderr, "\n");
     fprintf(stderr, "Input control:\n");
-    fprintf(stderr, "  -i\tread file list from <infile> "
+    fprintf(stderr, "  -i                   read file list from <infile> "
         "(stdin if '-' is specified)\n");
-    fprintf(stderr, "  -a\tinput contains arbitrary values "
+    fprintf(stderr, "  -a, --arbitrary      input contains arbitrary values "
         "(do not crawl filesystem)\n");
     fprintf(stderr, "\n");
     fprintf(stderr, "Output control:\n");
-    fprintf(stderr, "  -o\toutput partitions to <outfile> template "
-        "(stdout if '-' is specified)\n");
-    fprintf(stderr, "  -0\tend filenames with a null (\\0) character when "
-        "using option -o\n");
-    fprintf(stderr, "  -e\tadd ending slash to directories\n");
-    fprintf(stderr, "  -v\tverbose mode (may be specified more than once to "
-        "increase verbosity)\n");
+    fprintf(stderr, "  -o                   output partitions to <outfile> "
+        "template (stdout if '-' is\n");
+    fprintf(stderr, "                       specified)\n");
+    fprintf(stderr, "  -0                   end filenames with a null (\\0) "
+        "character when using\n");
+    fprintf(stderr, "                       option -o\n");
+    fprintf(stderr, "  -e                   add ending slash to directories\n");
+    fprintf(stderr, "  -P                   add parent directories when "
+        "closing intermediate\n");
+    fprintf(stderr, "                       partitions (needs -L)\n");
+    fprintf(stderr, "  -v, --verbose        verbose mode (may be specified "
+        "more than once to increase\n");
+    fprintf(stderr, "                       verbosity)\n");
     fprintf(stderr, "\n");
     fprintf(stderr, "Filesystem crawling control:\n");
-    fprintf(stderr, "  -l\tfollow symbolic links\n");
-    fprintf(stderr, "  -b\tdo not cross filesystem boundaries\n");
-    fprintf(stderr, "  -y\tinclude files matching <pattern> only (may be "
-        "specified more than once)\n");
+    fprintf(stderr, "  -l                   follow symbolic links\n");
+    fprintf(stderr, "  -b                   do not cross filesystem "
+        "boundaries\n");
+    fprintf(stderr, "  -y, --include        include files matching <pattern> "
+        "only (may be specified\n");
+    fprintf(stderr, "                       more than once)\n");
 #if defined(_HAS_FNM_CASEFOLD)
-    fprintf(stderr, "  -Y\tsame as -y, but ignore case\n");
+    fprintf(stderr, "  -Y                   same as -y, but ignore case\n");
 #endif
-    fprintf(stderr, "  -x\texclude files matching <pattern> (may be specified "
-        "more than once)\n");
+    fprintf(stderr, "  -x, --exclude        exclude files matching <pattern> "
+        "(may be specified more\n");
+    fprintf(stderr, "                       than once)\n");
 #if defined(_HAS_FNM_CASEFOLD)
-    fprintf(stderr, "  -X\tsame as -x, but ignore case\n");
+    fprintf(stderr, "  -X                   same as -x, but ignore case\n");
 #endif
     fprintf(stderr, "\n");
     fprintf(stderr, "Directory handling:\n");
-    fprintf(stderr, "  -z\tpack empty directories too "
+    fprintf(stderr, "  -z                   pack empty directories too "
         "(default: pack files only)\n");
-    fprintf(stderr, "  -zz\ttreat un-readable directories as empty\n");
-    fprintf(stderr, "  -zzz\tpack all directories (as empty)\n");
-    fprintf(stderr, "  -Z\tpack un-readable directories in separate "
-        "partitions\n");
-    fprintf(stderr, "  -d\tpack directories instead of files after a certain "
-        "<depth>\n");
-    fprintf(stderr, "  -D\tpack leaf directories (i.e. containing files only, "
-        "implies -z)\n");
-    fprintf(stderr, "  -E\tpack directories instead of files (implies -D)\n");
+    fprintf(stderr, "  -zz                  treat un-readable directories as "
+        "empty\n");
+    fprintf(stderr, "  -zzz                 pack all directories (as empty)\n");
+    fprintf(stderr, "  -Z                   pack un-readable directories in "
+        "separate partitions\n");
+    fprintf(stderr, "  -d                   pack directories instead of files "
+        "after a certain <depth>\n");
+    fprintf(stderr, "  -D, --leaf-dirs      pack leaf directories (i.e. "
+        "containing files only,\n");
+    fprintf(stderr, "                       implies -z)\n");
+    fprintf(stderr, "  -E, --dirs-only      pack directories instead of files "
+        "(implies -D)\n");
     fprintf(stderr, "\n");
     fprintf(stderr, "Live mode:\n");
-    fprintf(stderr, "  -L\tlive mode: generate partitions during filesystem "
-        "crawling\n");
-    fprintf(stderr, "  -S\tdo not pack files bigger than specified maximum "
-        "partition size\n");
-    fprintf(stderr, "    \tbut print them to stdout instead (needs -L and "
-        "-s)\n");
-    fprintf(stderr, "  -w\tpre-partition hook: execute <cmd> at partition "
-        "start\n");
-    fprintf(stderr, "  -W\tpost-partition hook: execute <cmd> at partition "
-        "end\n");
+    fprintf(stderr, "  -L, --live           live mode: generate partitions "
+        "during filesystem crawling\n");
+    fprintf(stderr, "  -S                   do not pack files bigger than "
+        "specified maximum partition\n");
+    fprintf(stderr, "                       size but print them to stdout "
+        "instead (needs -L and -s)\n");
+    fprintf(stderr, "  -w, --pre-part-cmd   pre-partition hook: execute <cmd> "
+        "at partition start\n");
+    fprintf(stderr, "  -W, --post-part-cmd  post-partition hook: execute <cmd> "
+        "at partition end\n");
+    fprintf(stderr, "  -R, --post-run-cmd   post-run hook: execute <cmd> "
+        "before exiting\n");
     fprintf(stderr, "\n");
     fprintf(stderr, "Size handling:\n");
-    fprintf(stderr, "  -p\tpreload each partition with <num> bytes\n");
-    fprintf(stderr, "  -q\toverload each file with <num> bytes\n");
-    fprintf(stderr, "  -r\tround each file size up to next <num> bytes "
-        "multiple\n");
+    fprintf(stderr, "  -p                   preload each partition with <num> "
+        "bytes\n");
+    fprintf(stderr, "  -q                   overload each file with <num> "
+        "bytes\n");
+    fprintf(stderr, "  -r                   round each file size up to next "
+        "<num> bytes multiple\n");
     fprintf(stderr, "\n");
     fprintf(stderr, "Example: fpart -n 3 -o var-parts /var\n");
     fprintf(stderr, "\n");
+#if !defined(HAVE_GETOPT_LONG)
+    fprintf(stderr, "Note: Long options are not supported on this platform.\n");
+#endif
     fprintf(stderr, "Please report bugs to Ganael LAPLANCHE "
         "<ganael.laplanche@martymac.org>\n");
+    fprintf(stderr, "WWW: http://contribs.martymac.org\n");
     return;
 }
 
@@ -169,15 +232,16 @@ usage(void)
    value) and update file entries (head)
    - returns != 0 if a critical error occurred
    - returns with head set to the last element added
-   - updates totalfiles with the number of elements added */
+   - updates main_status.total_size and main_status.total_num_files with
+     the number of elements added */
 static int
-handle_argument(char *argument, fnum_t *totalfiles, struct file_entry **head,
-    struct program_options *options)
+handle_argument(char *argument, struct file_entry **head,
+    struct program_options *options, struct program_status *status)
 {
     assert(argument != NULL);
-    assert(totalfiles != NULL);
     assert(head != NULL);
     assert(options != NULL);
+    assert(status != NULL);
 
     if(options->arbitrary_values == OPT_ARBITRARYVALUES) {
     /* handle arbitrary values */
@@ -188,11 +252,8 @@ handle_argument(char *argument, fnum_t *totalfiles, struct file_entry **head,
         )
 
         if(sscanf(argument, "%ju %[^\n]", &input_size, input_path) == 2) {
-            int handled = handle_file_entry(head, input_path, input_size,
-                0, options); /* entry_errno irrelevant here */
-            if(handled == 0)
-                (*totalfiles)++;
-            else if(handled < 0) {
+            if(handle_file_entry(head, input_path, input_size,
+                   0, options, status) /* entry_errno irrelevant here */ < 0) {
                 fprintf(stderr, "%s(): cannot add file entry\n", __func__);
                 free(input_path);
                 return (1);
@@ -213,14 +274,9 @@ handle_argument(char *argument, fnum_t *totalfiles, struct file_entry **head,
             return (1);
         )
         snprintf(input_path, malloc_size, "%s", argument);
-    
+
         /* remove multiple ending slashes */
-        while((input_path_len > 1) &&
-            (input_path[input_path_len - 1] == '/')  &&
-            (input_path[input_path_len - 2] == '/')) {
-            input_path[input_path_len - 1] = '\0';
-            input_path_len--;
-        }
+        cleanslash_path(input_path);
 
         /* crawl path */
         if(input_path[0] != '\0') {
@@ -228,7 +284,7 @@ handle_argument(char *argument, fnum_t *totalfiles, struct file_entry **head,
             fprintf(stderr, "init_file_entries(): examining %s\n",
                 input_path);
 #endif
-            if(init_file_entries(input_path, head, totalfiles, options) != 0) {
+            if(init_file_entries(input_path, head, options, status) != 0) {
                 fprintf(stderr, "%s(): cannot initialize file entries\n",
                     __func__);
                 free(input_path);
@@ -266,11 +322,9 @@ handle_options(struct program_options *options, int *argcp, char ***argvp)
     extern char *optarg;
     extern int optind;
     int ch;
-    while((ch = getopt(*argcp, *argvp,
-#if defined(_HAS_FNM_CASEFOLD)
-        "hVn:f:s:i:ao:0evlby:Y:x:X:zZd:DELSw:W:p:q:r:"
-#else
-        "hVn:f:s:i:ao:0evlby:x:zZd:DELSw:W:p:q:r:"
+    while((ch = GETOPT(*argcp, *argvp, OPTIONS
+#if defined(HAVE_GETOPT_LONG)
+        , long_options, NULL
 #endif
         )) != -1) {
         switch(ch) {
@@ -360,6 +414,9 @@ handle_options(struct program_options *options, int *argcp, char ***argvp)
             case 'e':
                 options->add_slash = OPT_ADDSLASH;
                 break;
+            case 'P':
+                options->add_parents = OPT_ADDPARENTS;
+                break;
             case 'v':
                 options->verbose++;
                 break;
@@ -462,6 +519,21 @@ handle_options(struct program_options *options, int *argcp, char ***argvp)
                 snprintf(options->post_part_hook, malloc_size, "%s", optarg);
                 break;
             }
+            case 'R':
+            {
+                /* check for empty argument */
+                size_t malloc_size = strlen(optarg) + 1;
+                if(malloc_size <= 1)
+                    break;
+                /* replace previous hook if '-R' specified multiple times */
+                if(options->post_run_hook != NULL)
+                    free(options->post_run_hook);
+                if_not_malloc(options->post_run_hook, malloc_size,
+                    return (FPART_OPTS_NOK | FPART_OPTS_EXIT);
+                )
+                snprintf(options->post_run_hook, malloc_size, "%s", optarg);
+                break;
+            }
             case 'p':
             {
                 uintmax_t preload_size = str_to_uintmax(optarg, 1);
@@ -548,6 +620,14 @@ handle_options(struct program_options *options, int *argcp, char ***argvp)
         return (FPART_OPTS_USAGE | FPART_OPTS_NOK | FPART_OPTS_EXIT);
     }
 
+    /* option -P (needs '-L') */
+    if((options->add_parents == OPT_ADDPARENTS) &&
+        (options->live_mode == OPT_NOLIVEMODE)) {
+        fprintf(stderr,
+            "Option -P can only be used with options -L.\n");
+        return (FPART_OPTS_USAGE | FPART_OPTS_NOK | FPART_OPTS_EXIT);
+    }
+
     /* We do not want to mix -E and -d as directory sizes are computed
        differently for those options: -E produces a single-depth total while -d
        computes a recursive total */
@@ -564,7 +644,8 @@ handle_options(struct program_options *options, int *argcp, char ***argvp)
 
     if((options->live_mode == OPT_NOLIVEMODE) &&
         ((options->pre_part_hook != NULL) ||
-        (options->post_part_hook != NULL))) {
+        (options->post_part_hook != NULL) ||
+        (options->post_run_hook != NULL))) {
         fprintf(stderr,
             "Hooks can only be used with option -L.\n");
         return (FPART_OPTS_USAGE | FPART_OPTS_NOK | FPART_OPTS_EXIT);
@@ -603,7 +684,12 @@ handle_options(struct program_options *options, int *argcp, char ***argvp)
 
 int main(int argc, char **argv)
 {
-    fnum_t totalfiles = 0;
+    /* Status */
+    struct program_status main_status = {
+        0,    /* total partitions size created so far */
+        0,    /* total number of files added so far */
+        0     /* total number of partitions created so far */
+    };
 
 /******************
   Handle options
@@ -666,8 +752,8 @@ int main(int argc, char **argv)
             if((line_end_p = strchr(line, '\n')) != NULL)
                 *line_end_p = '\0';
 
-            if(handle_argument(line, &totalfiles, &head, &options) != 0) {
-                uninit_file_entries(head, &options);
+            if(handle_argument(line, &head, &options, &main_status) != 0) {
+                uninit_file_entries(head, &options, &main_status);
                 uninit_options(&options);
                 exit(EXIT_FAILURE);
             }
@@ -682,8 +768,7 @@ int main(int argc, char **argv)
         }
 
         /* cleanup */
-        if(in_fp != NULL)
-            fclose(in_fp);
+        fclose(in_fp);
     }
 
 /******************
@@ -693,70 +778,69 @@ int main(int argc, char **argv)
     /* now, work on each path provided as arguments */
     int i;
     for(i = 0 ; i < argc ; i++) {
-        if(handle_argument(argv[i], &totalfiles, &head, &options) != 0) {
-            uninit_file_entries(head, &options);
+        if(handle_argument(argv[i], &head, &options, &main_status) != 0) {
+            uninit_file_entries(head, &options, &main_status);
             uninit_options(&options);
             exit(EXIT_FAILURE);
         }
     }
 
+    /* come back to the first element */
+    rewind_list(head);
+
 /****************
   Display status
 *****************/
 
-    /* come back to the first element */
-    rewind_list(head);
-
     /* no file found or live mode */
-    if((totalfiles == 0) || (options.live_mode == OPT_LIVEMODE)) {
-        uninit_file_entries(head, &options);
-        /* display status */
+    if((main_status.total_num_files == 0) || (options.live_mode == OPT_LIVEMODE)) {
+        uninit_file_entries(head, &options, &main_status);
+        /* display final summary */
         if(options.verbose >= OPT_VERBOSE)
-            fprintf(stderr, "%ju file(s) found.\n", totalfiles);
+            display_final_summary(main_status.total_num_parts,
+            main_status.total_size, main_status.total_num_files);
+
         uninit_options(&options);
         exit(EXIT_SUCCESS);
     }
 
-    /* display status */
-    if(options.verbose >= OPT_VERBOSE) {
-        fprintf(stderr, "%ju file(s) found.\n", totalfiles);
-        fprintf(stderr, "Sorting entries...\n");
-    }
-
 /************************************************
   Sort entries with a fixed number of partitions
 *************************************************/
 
+    if(options.verbose >= OPT_VERBOSE) {
+        fprintf(stderr, "Sorting entries...\n");
+    }
+
     /* our list of partitions */
     struct partition *part_head = NULL;
-    pnum_t num_parts = options.num_parts;
 
     /* sort files with a fixed size of partitions */
     if(options.num_parts != DFLT_OPT_NUM_PARTS) {
         /* create a fixed-size array of pointers to sort */
         struct file_entry **file_entry_p = NULL;
 
-        if_not_malloc(file_entry_p, sizeof(struct file_entry *) * totalfiles,
-            uninit_file_entries(head, &options);
+        if_not_malloc(file_entry_p, sizeof(struct file_entry *) * main_status.total_num_files,
+            uninit_file_entries(head, &options, &main_status);
             uninit_options(&options);
             exit(EXIT_FAILURE);
         )
 
         /* initialize array */
-        init_file_entry_p(file_entry_p, totalfiles, head);
+        init_file_entry_p(file_entry_p, main_status.total_num_files, head);
     
         /* sort array */
-        qsort(&file_entry_p[0], totalfiles, sizeof(struct file_entry *),
+        qsort(&file_entry_p[0], main_status.total_num_files, sizeof(struct file_entry *),
             &sort_file_entry_p);
     
         /* create a double_linked list of partitions
            which will hold dispatched files */
-        if(add_partitions(&part_head, options.num_parts, &options) != 0) {
+        if(add_partitions(&part_head, options.num_parts, &options, &main_status) != 0) {
             fprintf(stderr, "%s(): cannot init list of partitions\n",
                 __func__);
             uninit_partitions(part_head);
             free(file_entry_p);
-            uninit_file_entries(head, &options);
+            uninit_file_entries(head, &options, &main_status);
             uninit_options(&options);
             exit(EXIT_FAILURE);
         }
@@ -765,24 +849,24 @@ int main(int argc, char **argv)
     
         /* dispatch files */
         if(dispatch_file_entry_p_by_size
-            (file_entry_p, totalfiles, part_head, options.num_parts) != 0) {
+            (file_entry_p, main_status.total_num_files, part_head, options.num_parts) != 0) {
             fprintf(stderr, "%s(): unable to dispatch file entries\n",
                 __func__);
             uninit_partitions(part_head);
             free(file_entry_p);
-            uninit_file_entries(head, &options);
+            uninit_file_entries(head, &options, &main_status);
             uninit_options(&options);
             exit(EXIT_FAILURE);
         }
     
         /* re-dispatch empty files */
         if(dispatch_empty_file_entries
-            (head, totalfiles, part_head, options.num_parts) != 0) {
+            (head, main_status.total_num_files, part_head, options.num_parts) != 0) {
             fprintf(stderr, "%s(): unable to dispatch empty file entries\n",
                 __func__);
             uninit_partitions(part_head);
             free(file_entry_p);
-            uninit_file_entries(head, &options);
+            uninit_file_entries(head, &options, &main_status);
             uninit_options(&options);
             exit(EXIT_FAILURE);
         }
@@ -798,13 +882,13 @@ int main(int argc, char **argv)
     /* sort files with a file number or size limit per-partitions.
        In this case, partitions are dynamically-created */
     else {
-        if((num_parts = dispatch_file_entries_by_limits
+        if(dispatch_file_entries_by_limits
             (head, &part_head, options.max_entries, options.max_size,
-            &options)) == 0) {
+            &options, &main_status) == 0) {
             fprintf(stderr, "%s(): unable to dispatch file entries\n",
                 __func__);
             uninit_partitions(part_head);
-            uninit_file_entries(head, &options);
+            uninit_file_entries(head, &options, &main_status);
             uninit_options(&options);
             exit(EXIT_FAILURE);
         }
@@ -818,21 +902,27 @@ int main(int argc, char **argv)
   Print result and exit
 ************************/
 
-    /* print result summary */
+    /* print partitions summary */
     print_partitions(part_head, &options);
 
+    /* display final summary */
+    if(options.verbose >= OPT_VERBOSE) {
+        display_final_summary(main_status.total_num_parts,
+            main_status.total_size, main_status.total_num_files);
+    }
+
     if(options.verbose >= OPT_VERBOSE)
         fprintf(stderr, "Writing output lists...\n");
 
     /* print file entries */
-    print_file_entries(head, part_head, num_parts, &options);
+    print_file_entries(head, part_head, main_status.total_num_parts, &options);
 
     if(options.verbose >= OPT_VERBOSE)
         fprintf(stderr, "Cleaning up...\n");
 
     /* free stuff */
     uninit_partitions(part_head);
-    uninit_file_entries(head, &options);
+    uninit_file_entries(head, &options, &main_status);
     uninit_options(&options);
     exit(EXIT_SUCCESS);
 }
