@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2011-2022 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
+ * Copyright (c) 2011-2024 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -27,9 +27,18 @@
 #ifndef _FPART_H
 #define _FPART_H
 
-#define FPART_VERSION "1.5.1"
+#include "types.h"
+
+#define FPART_VERSION "1.6.0"
 
 /* maximum input line length, including '\n' and '\0' */
 #define MAX_LINE_LENGTH 2048
 
+/* Main program status */
+struct program_status {
+    fsize_t total_size;          /* total partitions size created so far */
+    fnum_t total_num_files;      /* total number of files added so far */
+    pnum_t total_num_parts;      /* total number of partitions created so far */
+};
+
 #endif /* _FPART_H */
