@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2011-2022 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
+ * Copyright (c) 2011-2024 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -51,6 +51,8 @@ init_options(struct program_options *options)
            (DFLT_OPT_OUT0 == OPT_OUT0));
     assert((DFLT_OPT_ADDSLASH == OPT_NOADDSLASH) ||
            (DFLT_OPT_ADDSLASH == OPT_ADDSLASH));
+    assert((DFLT_OPT_ADDPARENTS == OPT_NOADDPARENTS) ||
+           (DFLT_OPT_ADDPARENTS == OPT_ADDPARENTS));
     assert((DFLT_OPT_VERBOSE == OPT_NOVERBOSE) ||
            (DFLT_OPT_VERBOSE == OPT_VERBOSE) ||
            (DFLT_OPT_VERBOSE == OPT_VVERBOSE));
@@ -86,6 +88,7 @@ init_options(struct program_options *options)
     options->out_filename = NULL;
     options->out_zero = DFLT_OPT_OUT0;
     options->add_slash = DFLT_OPT_ADDSLASH;
+    options->add_parents = DFLT_OPT_ADDPARENTS;
     options->verbose = DFLT_OPT_VERBOSE;
     options->follow_symbolic_links = DFLT_OPT_FOLLOWSYMLINKS;
     options->cross_fs_boundaries = DFLT_OPT_CROSSFSBOUNDARIES;
@@ -106,6 +109,7 @@ init_options(struct program_options *options)
     options->skip_big = DFLT_OPT_SKIPBIG;
     options->pre_part_hook = NULL;
     options->post_part_hook = NULL;
+    options->post_run_hook = NULL;
     options->preload_size = DFLT_OPT_PRELOAD_SIZE;
     options->overload_size = DFLT_OPT_OVERLOAD_SIZE;
     options->round_size = DFLT_OPT_ROUND_SIZE;
@@ -118,6 +122,8 @@ uninit_options(struct program_options *options)
     options->round_size = DFLT_OPT_ROUND_SIZE;
     options->overload_size = DFLT_OPT_OVERLOAD_SIZE;
     options->preload_size = DFLT_OPT_PRELOAD_SIZE;
+    if(options->post_run_hook != NULL)
+        free(options->post_run_hook);
     if(options->post_part_hook != NULL)
         free(options->post_part_hook);
     if(options->pre_part_hook != NULL)
@@ -144,6 +150,7 @@ uninit_options(struct program_options *options)
     options->cross_fs_boundaries = DFLT_OPT_CROSSFSBOUNDARIES;
     options->follow_symbolic_links = DFLT_OPT_FOLLOWSYMLINKS;
     options->verbose = DFLT_OPT_VERBOSE;
+    options->add_parents = DFLT_OPT_ADDPARENTS;
     options->add_slash = DFLT_OPT_ADDSLASH;
     options->out_zero = DFLT_OPT_OUT0;
     if(options->out_filename != NULL)
